# frozen_string_literal: true

require "rspec/autorun"
require "pry"

require "erb"
require "fileutils"
require "pathname"

module GoGoGo
  class Project
    def initialize(name)
      @name = name
      @underscored_name = @name.tr("-", "_")
      @constant_name =
        @name.
        gsub(/-[_-]*(?![_-]|$)/) { "::" }.
        gsub(/([_-]+|(::)|^)(.|$)/) { $2.to_s + $3.upcase }
    end

    def create!
      template_path = File.expand_path("../template", __FILE__)

      # copy the template
      FileUtils.cp_r template_path, @name

      # rename the files
      FileUtils.mv(
        File.join(@name, "app.rb"),
        File.join(@name, "#{@underscored_name}.rb")
      )

      # apply templates
      [
        File.join(@name, "Guardfile"),
        File.join(@name, "#{@underscored_name}.rb"),
      ].each { |file| render_template_to_file file, binding }
    end

    private

    def render_template_to_file(template, context)
      t = File.read(template)

      File.open(template, "w") do |f|
        f << ERB.new(t, nil, "-").result(context)
      end
    end
  end
end

describe "GoGoGo" do
  before :each do
    @folder = "my_project"
  end

  after :each do
    FileUtils.rm_rf @folder
  end

  describe "::create" do
    it "creates project directory" do
      GoGoGo::Project.new(@folder).create!

      expect(
        File.exist?(File.join(@folder, ".rspec"))
      ).to be true

      expect(
        File.exist?(File.join(@folder, "my_project.rb"))
      ).to be true

      expect(
        File.exist?(File.join(@folder, "Gemfile"))
      ).to be true

      expect(
        File.exist?(File.join(@folder, "Guardfile"))
      ).to be true
    end
  end
end
